package com.diagbot.aggregate;

import com.diagbot.dto.LisResult;
import com.diagbot.dto.PacsResult;
import com.diagbot.enums.FeatureTypeEnum;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.facade.TransferFacade;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.SearchVo;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/11/14 16:11
 */
@Component
public class SearchItemAggregate {
    @Autowired
    private TransferFacade transferFacade;

    @DataProvider("assembleSearchData")
    public SearchVo assembleSearchData(@InvokeParameter("searchVo") SearchVo searchVo,
                                       @InvokeParameter("isConnect") Boolean isConnect,
                                       @DataConsumer("setLisArr") List<LisResult> lisArr,
                                       @DataConsumer("setPacsStr") String pacs,
                                       @DataConsumer("setDiag") String diag,
                                       @DataConsumer("setDiseaseName") String diseaseName,
                                       @DataConsumer("setLisOrder") String lisOrder,
                                       @DataConsumer("setPacsOrder") String pacsOrder) {
        if (isConnect) {
            //化验转公表项处理
            searchVo.setLisArr(lisArr);
            //下面转换仅限数据引擎模式
            if (searchVo.getSysType().equals(SysTypeEnum.LTAPI_SERVICE.getKey())) {
                //辅检预处理
                searchVo.setPacs(pacs);
                //诊断预处理
                searchVo.setDiag(diag);
                //治疗方案诊断转换
                searchVo.setDiseaseName(diseaseName);
                //化验开单项
                searchVo.setLisOrder(lisOrder);
                //辅检开单项
                searchVo.setPacsOrder(pacsOrder);
            }
        }
        return searchVo;
    }

    /**
     * 化验增加公表项
     *
     * @param searchVo
     * @return
     */
    @DataProvider("setLisArr")
    public List<LisResult> setLisArr(@InvokeParameter("searchVo") SearchVo searchVo) {
        List<LisResult> lisArr = searchVo.getLisArr();
        lisArr = transferFacade.addLisUniqueName(lisArr, searchVo.getHosCode());
        return lisArr;
    }

    /**
     * 辅检转内部项目名称
     *
     * @param searchVo
     * @return
     */
    @DataProvider("setPacsStr")
    public String setPacsStr(@InvokeParameter("searchVo") SearchVo searchVo) {
        String pacs = "";
        if (StringUtil.isNotBlank(searchVo.getPacs())) {
            pacs += searchVo.getPacs() + ";";
        }
        List<PacsResult> pacsArr = searchVo.getPacsArr();
        if (ListUtil.isNotEmpty(pacsArr)) {
            List<PacsResult> convertPacsArr = transferFacade.addPacsUniqueName(pacsArr, searchVo.getHosCode());
            for (PacsResult pacsResult : convertPacsArr) {
                String pacsResultString = "";
                if (StringUtils.isNotBlank(pacsResult.getUniqueName())) {
                    pacsResultString = pacsResult.getUniqueName() + ":";
                } else {
                    pacsResultString = pacsResult.getName() + ":";
                }
                pacsResultString += pacsResult.getResult() + ";";
                pacs += pacsResultString;
            }
        }
        return pacs;
    }

    /**
     * 诊断转内部名称
     *
     * @param searchVo
     * @return
     */
    @DataProvider("setDiag")
    public String setDiag(@InvokeParameter("searchVo") SearchVo searchVo) {
        String diag = "";
        if (StringUtil.isNotBlank(searchVo.getDiag())) {
            diag = transferFacade.strConvert(searchVo.getDiag(), searchVo.getHosCode(), FeatureTypeEnum.Feature_Type_Disease.getKey());
        }
        return diag;
    }

    /**
     * 获取治疗方案诊断转内部名称
     *
     * @param searchVo
     * @return
     */
    @DataProvider("setDiseaseName")
    public String setDiseaseName(@InvokeParameter("searchVo") SearchVo searchVo) {
        String diseaseName = "";
        if (StringUtil.isNotBlank(searchVo.getDiseaseName())) {
            diseaseName = transferFacade.strConvert(searchVo.getDiseaseName(), searchVo.getHosCode(), FeatureTypeEnum.Feature_Type_Disease.getKey());
        }
        return diseaseName;
    }

    /**
     * 化验开单项映射公表名称
     *
     * @param searchVo
     * @return
     */
    @DataProvider("setLisOrder")
    public String setLisOrder(@InvokeParameter("searchVo") SearchVo searchVo) {
        String lisOrder = "";
        if (StringUtil.isNotBlank(searchVo.getLisOrder())) {
            lisOrder = transferFacade.strConvert(searchVo.getLisOrder(), searchVo.getHosCode(), FeatureTypeEnum.Feature_Type_Lis.getKey());
        }
        return lisOrder;
    }

    /**
     * 辅检开单项转内部项目名称
     *
     * @param searchVo
     * @return
     */
    @DataProvider("setPacsOrder")
    public String setPacsOrder(@InvokeParameter("searchVo") SearchVo searchVo) {
        String pacsOrder = "";
        if (StringUtil.isNotBlank(searchVo.getPacsOrder())) {
            pacsOrder = transferFacade.strConvert(searchVo.getPacsOrder(), searchVo.getHosCode(), FeatureTypeEnum.Feature_Type_Pacs.getKey());
        }
        return pacsOrder;
    }
}