package com.diagbot.aggregate;

import com.diagbot.client.bean.AdverseReaction;
import com.diagbot.client.bean.Medicition;
import com.diagbot.client.bean.MedicitionClass;
import com.diagbot.dto.ConceptDetailDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.ConceptDetail;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.enums.PositionTypeEnum;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/10/24 10:01
 */
@Component
public class TreatmentAggregate {

    @DataProvider("setTreatmentAll")
    public Map<String, Object> setTreatmentAll(@InvokeParameter("treatmentMap") Map<String, Object> treatmentMap,
                                               @DataConsumer("setCommonTreatment") ConceptDetailDTO commonTreatment,
                                               @DataConsumer("setSurgeryTreatment") ConceptDetailDTO surgeryTreatment,
                                               @DataConsumer("setDrugs") List<MedicitionClass> drugsList,
                                               @DataConsumer("setAdverseReactions") List<AdverseReaction> adverseReactionList) {
        if (treatmentMap == null) {
            treatmentMap = new LinkedHashMap<>();
        }
        treatmentMap.put("commonTreatment", commonTreatment);
        treatmentMap.put("surgeryTreatment", surgeryTreatment);
        treatmentMap.put("treatment", drugsList);
        treatmentMap.put("adverseReactions", adverseReactionList);

        return treatmentMap;
    }

    @DataProvider("setCommonTreatment")
    public ConceptDetailDTO setCommonTreatment(@InvokeParameter("diseaseName") String diseaseName,
                                               @InvokeParameter("hasConDetailMap") Map<String, Map<Long, List<ConceptDetail>>> hasConDetailMap) {
        ConceptDetailDTO conceptDetailDTO = null;
        if (hasConDetailMap != null && hasConDetailMap.get(diseaseName) != null
                && ListUtil.isNotEmpty(hasConDetailMap
                .get(diseaseName)
                .get(Long.valueOf(LexiconTypeEnum.DIAGNOSIS.getKey())))) {
            List<ConceptDetail> diseaseConDetailList
                    = hasConDetailMap.get(diseaseName).get(Long.valueOf(LexiconTypeEnum.DIAGNOSIS.getKey()));
            for (ConceptDetail conceptDetail : diseaseConDetailList) {
                List<String> positionList = Arrays.asList(conceptDetail.getPosition().split(",|，"));
                if (positionList.contains(String.valueOf(PositionTypeEnum.T3.getKey()))) {
                    conceptDetailDTO = new ConceptDetailDTO();
                    BeanUtil.copyProperties(conceptDetail, conceptDetailDTO);
                    break;
                }
            }
        }
        return conceptDetailDTO;
    }

    @DataProvider("setSurgeryTreatment")
    public ConceptDetailDTO setSurgeryTreatment(@InvokeParameter("diseaseName") String diseaseName,
                                                @InvokeParameter("hasConDetailMap") Map<String, Map<Long, List<ConceptDetail>>> hasConDetailMap) {
        ConceptDetailDTO conceptDetailDTO = null;
        if (hasConDetailMap != null && hasConDetailMap.get(diseaseName) != null
                && ListUtil.isNotEmpty(hasConDetailMap
                .get(diseaseName)
                .get(Long.valueOf(LexiconTypeEnum.DIAGNOSIS.getKey())))) {
            List<ConceptDetail> diseaseConDetailList
                    = hasConDetailMap.get(diseaseName).get(Long.valueOf(LexiconTypeEnum.DIAGNOSIS.getKey()));
            for (ConceptDetail conceptDetail : diseaseConDetailList) {
                List<String> positionList = Arrays.asList(conceptDetail.getPosition().split(",|，"));
                if (positionList.contains(String.valueOf(PositionTypeEnum.T4.getKey()))) {
                    conceptDetailDTO = new ConceptDetailDTO();
                    BeanUtil.copyProperties(conceptDetail, conceptDetailDTO);
                    break;
                }
            }
        }
        return conceptDetailDTO;
    }

    @DataProvider("setDrugs")
    public List<MedicitionClass> setDrugs(@InvokeParameter("drugsList") List<MedicitionClass> drugsList,
                                          @InvokeParameter("conceptMap") Map<String, Map<Long, Concept>> conceptMap,
                                          @InvokeParameter("hasConDetailMap") Map<String, Map<Long, List<ConceptDetail>>> hasConDetailMap) {
        if (ListUtil.isNotEmpty(drugsList)) {
            for (MedicitionClass medicitionClass : drugsList) {
                if (conceptMap != null
                        && conceptMap.get(medicitionClass.getBigdrugsName()) != null
                        && conceptMap.get(medicitionClass
                        .getBigdrugsName())
                        .containsKey(Long.valueOf(LexiconTypeEnum.DRUG_CATEGORY_BIG.getKey()))) {
                    Concept cateBigConcept = conceptMap
                            .get(medicitionClass.getBigdrugsName())
                            .get(Long.valueOf(LexiconTypeEnum.DRUG_CATEGORY_BIG.getKey()));
                    if (cateBigConcept != null) {
                        medicitionClass.setBigdrugsConceptId(cateBigConcept.getId());
                        if (hasConDetailMap != null
                                && hasConDetailMap.get(cateBigConcept.getLibName()) != null
                                && hasConDetailMap.get(cateBigConcept.getLibName())
                                .containsKey(Long.valueOf(LexiconTypeEnum.DRUG_CATEGORY_BIG.getKey()))) {
                            medicitionClass.setShowInfo("1");
                        } else {
                            medicitionClass.setShowInfo("0");
                        }
                    }
                }
                medicitionClass.setBigdrgusLibType(LexiconTypeEnum.DRUG_CATEGORY_BIG.getKey());
                medicitionClass.setBigdrugsType(ConceptTypeEnum.Drug_Category_Big.getKey());
                if (conceptMap != null
                        && conceptMap.get(medicitionClass.getSubdrugsName()) != null
                        && conceptMap.get(medicitionClass.getSubdrugsName())
                        .containsKey(Long.valueOf(LexiconTypeEnum.DRUG_CATEGORY_SMALL.getKey()))) {
                    Concept cateSmallConcept = conceptMap
                            .get(medicitionClass.getSubdrugsName())
                            .get(Long.valueOf(LexiconTypeEnum.DRUG_CATEGORY_SMALL.getKey()));
                    if (cateSmallConcept != null) {
                        medicitionClass.setSubdrugsConceptId(cateSmallConcept.getId());
                    }
                }
                medicitionClass.setSubdrugsLibType(LexiconTypeEnum.DRUG_CATEGORY_SMALL.getKey());
                medicitionClass.setSubdrugsType(ConceptTypeEnum.Drug_Category_Small.getKey());

                LinkedList<Medicition> medicitionList = medicitionClass.getMedicitionsList();
                for (Medicition medicition : medicitionList) {
                    if (conceptMap != null
                            && conceptMap.get(medicition.getMedicitionName()) != null
                            && conceptMap.get(medicition.getMedicitionName())
                            .containsKey(Long.valueOf(LexiconTypeEnum.DRUGS.getKey()))) {
                        Concept drugConcept = conceptMap
                                .get(medicition.getMedicitionName())
                                .get(Long.valueOf(LexiconTypeEnum.DRUGS.getKey()));
                        if (drugConcept != null) {
                            medicition.setConceptId(drugConcept.getId());
                        }
                    }
                    medicition.setLibType(LexiconTypeEnum.DRUGS.getKey());
                    medicition.setType(ConceptTypeEnum.Drug.getKey());
                    if (hasConDetailMap != null
                            && hasConDetailMap.get(medicition.getMedicitionName()) != null
                            && hasConDetailMap.get(medicition.getMedicitionName())
                            .containsKey(Long.valueOf(LexiconTypeEnum.DRUGS.getKey()))) {
                        medicition.setShowInfo("1");
                    } else {
                        medicition.setShowInfo("0");
                    }
                }
                medicitionClass.setMedicitionsList(medicitionList);
            }
        }
        return drugsList;
    }

    @DataProvider("setAdverseReactions")
    public List<AdverseReaction> setAdverseReactions(@InvokeParameter("adverseReactionList") List<AdverseReaction> adverseReactionList,
                                                     @InvokeParameter("conceptMap") Map<String, Map<Long, Concept>> conceptMap,
                                                     @InvokeParameter("hasConDetailMap") Map<String, Map<Long, List<ConceptDetail>>> hasConDetailMap,
                                                     @InvokeParameter("isChronic") Boolean isChronic) {
        if (isChronic) {
            //不良反应
            for (AdverseReaction adverseReaction : adverseReactionList) {
                if (conceptMap != null
                        && conceptMap.get(adverseReaction.getName()) != null
                        && conceptMap.get(adverseReaction.getName())
                        .containsKey(Long.valueOf(LexiconTypeEnum.SIDE_EFFECTS.getKey()))) {
                    Concept adConcept = conceptMap
                            .get(adverseReaction.getName())
                            .get(Long.valueOf(LexiconTypeEnum.SIDE_EFFECTS.getKey()));
                    if (adConcept != null) {
                        adverseReaction.setConceptId(adConcept.getId());
                    }
                }
                adverseReaction.setLibType(LexiconTypeEnum.SIDE_EFFECTS.getKey());
                adverseReaction.setType(ConceptTypeEnum.SIDE_EFFECTS.getKey());
                if (hasConDetailMap != null
                        && hasConDetailMap.get(adverseReaction.getName()) != null
                        && hasConDetailMap.get(adverseReaction.getName())
                        .containsKey(Long.valueOf(LexiconTypeEnum.SIDE_EFFECTS.getKey()))) {
                    adverseReaction.setShowInfo("1");
                } else {
                    adverseReaction.setShowInfo("0");
                }
            }
        }
        return adverseReactionList;
    }
}
