package com.diagbot.facade;

import com.alibaba.fastjson.JSONObject;
import com.diagbot.client.bean.ResponseData;
import com.diagbot.dto.PushDTO;
import com.diagbot.enums.FeatureTypeEnum;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.SearchVo;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/5/7 16:12
 */
@Component
public class PushFacade {
    @Autowired
    private ClinicalFacade clinicalFacade;
    @Autowired
    private TreatmentFacade treatmentFacade;
    @Autowired
    private TransferFacade transferFacade;
    @Autowired
    DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;

    /**
     * 推理接口
     *
     * @param searchVo
     * @return ResponseData
     */
    public PushDTO pushInner(SearchVo searchVo) {
        PushDTO pushDTO = new PushDTO();
        //是否对接
        Boolean isConnect = transferFacade.isConnect(searchVo.getHosCode(), searchVo.getSysType());
        //入参预处理
        searchVo = precSearchData(searchVo, isConnect);
        ResponseData data = clinicalFacade.processClinicalData(searchVo);

        String featureType = searchVo.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("featureTypeSet", featureTypeSet);
            invokeParams.put("data", data);
            invokeParams.put("isConnect", isConnect);
            invokeParams.put("hosCode", searchVo.getHosCode());
            pushDTO
                    = dataBeanAggregateQueryFacade.get("setAll", invokeParams, PushDTO.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return pushDTO;
    }

    /**
     * 获取管理评估大数据推理内容
     *
     * @param searchVo
     * @return
     */
    public Map<String, JSONObject> getManagementEvaluationContent(SearchVo searchVo) {
        ResponseData data = clinicalFacade.processClinicalData(searchVo);

        String featureType = searchVo.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_ManagementEvaluation.getKey()))) {
            return data.getManagementEvaluation();
        } else {
            return null;
        }
    }

    /**
     * 获取量表推理内容
     *
     * @param searchVo
     * @return
     */
    public Map<String, Object> scale(SearchVo searchVo) {
        Map<String, Object> data = clinicalFacade.scale(searchVo);
        return data;
    }

    /**
     * 获取治疗方案
     *
     * @param searchVo
     * @return
     */
    public Map<String, Object> getTreatment(SearchVo searchVo) {
        //记录外部诊断名称
        String diseaseName = searchVo.getDiseaseName();
        //是否对接
        Boolean isConnect = transferFacade.isConnect(searchVo.getHosCode(), searchVo.getSysType());
        //入参预处理
        searchVo = precSearchData(searchVo, isConnect);
        ResponseData data = clinicalFacade.processClinicalData(searchVo);
        Map<String, JSONObject> treat = data.getTreat();
        if (StringUtil.isBlank(searchVo.getDiseaseName())) {
            throw new CommonException(CommonErrorCode.PARAM_ERROR, "请输入需获取治疗方案的诊断名称");
        }
        Map<String, Object> treatmentMap
                = treatmentFacade.getTreatment(treat, searchVo.getDiseaseName(), searchVo.getDisType());
        //数据引擎模式下，诊断名称转换成外部名称
        if (isConnect && searchVo.getSysType().equals(SysTypeEnum.LTAPI_SERVICE.getKey())) {
            treatmentMap.put("diseaseName", diseaseName);
        }
        return treatmentMap;
    }

    /**
     * searchData入参预处理
     *
     * @param searchVo
     * @param isConnect
     * @return
     */
    public SearchVo precSearchData(SearchVo searchVo, Boolean isConnect) {
        try {
            Map<String, Object> inputParams = new HashMap<>();
            inputParams.put("searchVo", searchVo);
            inputParams.put("isConnect", isConnect);
            searchVo
                    = dataBeanAggregateQueryFacade.get("assembleSearchData", inputParams, SearchVo.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return searchVo;
    }
}