package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.PushFacade;
import com.diagbot.vo.PushVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/5/13 18:00
 */
@RestController
@RequestMapping("/push")
@Api(value = "推理相关API", tags = { "数据服务模式-推理相关API" })
@SuppressWarnings("unchecked")
public class PushController {
    @Autowired
    PushFacade pushFacade;

    @ApiOperation(value = "数据服务模式-推理[by:zhaops]",
            notes = "age： 年龄(必填)<br>" +
                    "sex：性别(必填) 1：男，2：女<br>" +
                    "symptom：症状,String<br>" +
                    "other：其它史,String<br>" +
                    "vital：查体,String<br>" +
                    "lis：化验,Array<br>" +
                    "lis：name：套餐名称(必填),String<br>" +
                    "lis：detailName：化验项名称(必填),String<br>" +
                    "lis：uniqueName：公表名称,String<br>" +
                    "lis：value：值,Double<br>" +
                    "lis：maxValue：最大值,Double<br>" +
                    "lis：minValue：最小值,Double<br>" +
                    "lis：units：单位,String<br>" +
                    "lis：otherValue：其他结果，包括阴性阳性，文字描述等,String<br>" +
                    "lis：source：数据来源（必填）,Integer<br>" +
                    "pacs：辅检,String<br>" +
                    "diag：诊断,String<br>" +
                    "featureType：类型(必填)，1：症状，3：其他史，4：查体结果，42：查体指标，5：化验，6：辅检，7：诊断,22-指标,String<br>" +
                    "diseaseName：取治疗方案的诊断,String<br>" +
                    "scaleName：量表名称,String<br>" +
                    "disType：诊断类型，1-慢病，2-急诊，普通病不填,Integer<br>" +
                    "indications：指标结果,String<br>" +
                    "adverseReactions：不良反应,String<br>" +
                    "lisString：化验纯文本,String<br>" +
                    "lisOrder：下单化验项，多项用分号隔开,String<br>" +
                    "pacsOrder：下单辅检项，多项用分号隔开,String<br>" +
                    "otherOrder：其他下单项，多项用分号隔开,String<br>")
    @PostMapping("/push")
    @SysLogger("push")
    public RespDTO<PushDTO> push(@RequestBody @Valid PushVO pushVO) {
        return RespDTO.onSuc(pushFacade.push(pushVO));
    }

    @ApiOperation(value = "数据服务模式-治疗方案[by:zhaops]",
            notes = "age： 年龄(必填)<br>" +
                    "sex：性别(必填)，1：男，2：女<br>" +
                    "symptom：症状,String<br>" +
                    "other：其它史,String<br>" +
                    "vital：查体,String<br>" +
                    "lis：化验,Array<br>" +
                    "lis：name：套餐名称(必填),String<br>" +
                    "lis：detailName：化验项名称(必填),String<br>" +
                    "lis：uniqueName：公表名称,String<br>" +
                    "lis：value：值,Double<br>" +
                    "lis：maxValue：最大值,Double<br>" +
                    "lis：minValue：最小值,Double<br>" +
                    "lis：units：单位,String<br>" +
                    "lis：otherValue：其他结果，包括阴性阳性，文字描述等,String<br>" +
                    "lis：source：数据来源（必填）,Integer<br>" +
                    "pacs：辅检,String<br>" +
                    "diag：诊断,String<br>" +
                    "featureType：类型(必填)，8：治疗方案,String<br>" +
                    "diseaeName：取治疗方案的诊断,String<br>" +
                    "disType：诊断类型，1-慢病，2-急诊，普通病不填,Integer<br>" +
                    "indications：指标结果,String<br>" +
                    "adverseReactions：不良反应,String<br>" +
                    "hosCode：医院编码,String<br>" +
                    "lisString：化验纯文本,String<br>" +
                    "lisOrder：下单化验项，多项用分号隔开,String<br>" +
                    "pacsOrder：下单辅检项，多项用分号隔开,String<br>" +
                    "otherOrder：其他下单项，多项用分号隔开,String<br>")
    @PostMapping("/pushTreatment")
    @SysLogger("pushTreatment")
    public RespDTO<Map<String, Object>> pushTreatment(@RequestBody @Valid PushVO pushVO) {
        return RespDTO.onSuc(pushFacade.pushTreatment(pushVO));
    }

    @ApiOperation(value = "数据服务模式-量表[by:zhaops]",
            notes = "age： 年龄(必填)<br>" +
                    "sex：性别(必填)，1：男，2：女<br>" +
                    "symptom：症状,String<br>" +
                    "other：其它史,String<br>" +
                    "vital：查体,String<br>" +
                    "lis：化验,Array<br>" +
                    "lis：name：套餐名称(必填),String<br>" +
                    "lis：detailName：化验项名称(必填),String<br>" +
                    "lis：uniqueName：公表名称,String<br>" +
                    "lis：value：值,Double<br>" +
                    "lis：maxValue：最大值,Double<br>" +
                    "lis：minValue：最小值,Double<br>" +
                    "lis：units：单位,String<br>" +
                    "lis：otherValue：其他结果，包括阴性阳性，文字描述等,String<br>" +
                    "lis：source：数据来源（必填）,Integer<br>" +
                    "pacs：辅检,String<br>" +
                    "diag：诊断,String<br>" +
                    "featureType：类型(必填)，21：量表,String<br>" +
                    "diseaeName：取治疗方案的诊断,String<br>" +
                    "scaleName：量表名称,String<br>" +
                    "disType：诊断类型，1-慢病，2-急诊，普通病不填,Integer<br>" +
                    "indications：指标结果,String<br>" +
                    "adverseReactions：不良反应,String<br>" +
                    "hosCode：医院编码,String<br>" +
                    "lisString：化验纯文本,String<br>" +
                    "lisOrder：下单化验项，多项用分号隔开,String<br>" +
                    "pacsOrder：下单辅检项，多项用分号隔开,String<br>" +
                    "otherOrder：其他下单项，多项用分号隔开,String<br>")
    @PostMapping("/pushScale")
    @SysLogger("pushScale")
    public RespDTO<Map<String, Object>> pushScale(@RequestBody @Valid PushVO pushVO) {
        return RespDTO.onSuc(pushFacade.pushScale(pushVO));
    }
}