package com.diagbot.facade;

import com.diagbot.client.AiptServiceClient;
import com.diagbot.client.bean.ResponseData;
import com.diagbot.client.bean.SearchData;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushKYJDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.enums.FeatureTypeEnum;
import com.diagbot.enums.LisSourceEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.LisResult;
import com.diagbot.vo.PushKYJVO;
import com.diagbot.vo.PushVO;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2018/11/20 11:18
 */
@Component
public class PushFacade {
    @Autowired
    private DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;
    @Autowired
    private TreatmentFacade treatmentFacade;
    @Autowired
    private AiptServiceClient aiptServiceClient;
    @Autowired
    private AssembleFacade assembleFacade;


    /**
     * 中间层接口
     *
     * @param searchData
     * @return ResponseData
     */
    public ResponseData pushAipt(SearchData searchData) {
        RespDTO<ResponseData> res = aiptServiceClient.aiptData(searchData);
        RespDTOUtil.respNGDeal(res, "中间层没有返回结果");
        return res.data;
    }

    /**
     * 中间层推理-症状……诊断、核心指标
     *
     * @param searchData
     * @return
     */
    public PushDTO pushInnerAipt(SearchData searchData) {
        RespDTO<PushDTO> res = aiptServiceClient.pushInner(searchData);
        RespDTOUtil.respNGDeal(res, "中间层没有返回结果");
        return res.data;
    }

    /**
     * 中间层获取治疗方案
     *
     * @param searchData
     * @return
     */
    public Map<String, Object> getTreatment(SearchData searchData) {
        RespDTO<Map<String, Object>> res = aiptServiceClient.getTreatment(searchData);
        RespDTOUtil.respNGDealCover(res, "中间层没有返回治疗方案");
        return res.data;
    }

    /**
     * 推理接口
     *
     * @param pushVO
     * @return
     */
    public PushDTO pushInner(PushVO pushVO) {
        PushDTO pushDTO = new PushDTO();
        SearchData aiSearchData = assembleFacade.assembleData(pushVO);
        PushDTO data = pushInnerAipt(aiSearchData);

        String featureType = pushVO.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));

        //推理结果返回科室
        pushDTO.setDept(data.getDept());

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("pushVO", pushVO);
            invokeParams.put("pushDTO", pushDTO);
            invokeParams.put("featureTypeSet", featureTypeSet);
            invokeParams.put("data", data);
            pushDTO
                    = dataBeanAggregateQueryFacade.get("setICSSAll", invokeParams, PushDTO.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        } finally {
            if (pushDTO == null) {
                pushDTO = new PushDTO();
                pushDTO.setDept(data.getDept());
            }
        }

        //其他史 概念+标签id
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Other.getKey()))) {
        }
        //辅检  标签列表
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Pacs.getKey()))) {
            pushDTO.setPacs(data.getPacs());
        }
        //诊断 返回分类+标签列表
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Disease.getKey()))) {
            pushDTO.setDis(data.getDis());
        }
        //推送指标 featuretype=22
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.MedicalIndication.getKey()))) {
            pushDTO.setMedicalIndications(data.getMedicalIndications());
        }
        return pushDTO;
    }

    /**
     * 治疗方案推送
     *
     * @param pushVO
     * @return
     */
    public Map<String, Object> pushTreatment(PushVO pushVO) {
        SearchData searchData = assembleFacade.assembleData(pushVO);
        Map<String, Object> data = getTreatment(searchData);
        data = treatmentFacade.getTreatment(data, pushVO.getPatientId());
        return data;
    }

    /**
     * 推理接口-快易检
     *
     * @param pushKYJVO
     * @return
     */
    public PushKYJDTO pushKYJ(PushKYJVO pushKYJVO) {
        PushVO pushVO = new PushVO();
        pushVO.setAge(pushKYJVO.getAge());
        pushVO.setSex(pushKYJVO.getSex());
        pushVO.setFeatureType(pushKYJVO.getFeatureType());

        //化验结果
        List<LisResult> lisResultList = pushKYJVO.getLisResultList();
        for (LisResult lisResult : lisResultList) {
            lisResult.setSource(LisSourceEnum.Outer.getKey());
        }
        pushVO.setLis(lisResultList);

        SearchData searchData = assembleFacade.assembleData(pushVO);
        ResponseData responseData = pushAipt(searchData);
        PushKYJDTO pushKYJDTO = new PushKYJDTO();
        pushKYJDTO.setLabs(responseData.getLabs());
        pushKYJDTO.setDis(responseData.getDis());
        return pushKYJDTO;
    }
}