package com.diagbot.aop;

import com.diagbot.enums.ProductTypeEnum;
import com.diagbot.log.entity.BiRecord;
import com.diagbot.rabbit.MySender;
import com.diagbot.util.AopUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @Description: BI日志结果特殊处理拦截切面
 * @author: gaodm
 * @time: 2019/11/12 17:26
 */
@Aspect
@Component
public class BiLoggerResultAspect {
    @Autowired
    private MySender mySender;

    @Pointcut("@annotation(com.diagbot.annotation.BiLoggerResult)")
    public void biResultPointCut() {
    }

    @AfterReturning(value = "biResultPointCut()", returning = "keys")
    public void saveBiRecord(JoinPoint joinPoint, Object keys) {
        BiRecord biRecord = AopUtil.BiLoggerResultAspect(joinPoint, keys, ProductTypeEnum.LTAPI.getKey());
        if (null == biRecord) {
            return;
        }
        //保存BI日志
        mySender.outputBiLogSend(biRecord);
    }

    @AfterThrowing(pointcut = "biResultPointCut()", throwing = "ex")
    public void saveBiRecordThrow(JoinPoint joinPoint, Throwable ex) {
        BiRecord biRecord = AopUtil.BiLoggerResultAspectThrow(joinPoint, ex, ProductTypeEnum.LTAPI.getKey());
        if (null == biRecord) {
            return;
        }
        //保存BI日志
        mySender.outputBiLogSend(biRecord);
    }
}

