package com.diagbot.facade;

import com.diagbot.dto.GetTopPatientInfoDTO;
import com.diagbot.dto.PatientInfoDTO;
import com.diagbot.dto.SignInDTO;
import com.diagbot.entity.PatientInfo;
import com.diagbot.service.impl.PatientInfoServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.vo.GetTopPatientInfoVO;
import com.diagbot.vo.PatientInfoVO;
import com.diagbot.vo.SignInVO;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 患者业务逻辑
 * @author: wangyu
 * @time: 2018/11/19 13:19
 */
@Component
public class PatientInfoFacade extends PatientInfoServiceImpl {

    /*@Autowired
    private DoctorPageModeFacade doctorPageModeFacade;*/
    /*@Autowired
    private DeptInfoFacade deptInfoFacade;*/

    /**
     * 获取患者信息
     *
     * @param patientInfoVO
     * @return
     */
    public PatientInfoDTO getPatientInfo(PatientInfoVO patientInfoVO) {
        PatientInfoDTO patientInfoDTO = this.getPatientInfos(patientInfoVO.getPatientCode(),
                patientInfoVO.getHospitalCode());
        return patientInfoDTO;
    }

    /**
     * 页面顶部病人医生科室信息查询
     *
     * @param getTopPatientInfoVO
     * @return
     */
    public GetTopPatientInfoDTO getTopPatientInfo(@RequestBody GetTopPatientInfoVO getTopPatientInfoVO) {
        GetTopPatientInfoDTO getTopPatientInfoDTO = baseMapper.getTopPatientInfo(getTopPatientInfoVO);
        getTopPatientInfoDTO.setSystemTime(DateUtil.now());
        getTopPatientInfoDTO.setRecordId(getTopPatientInfoVO.getRecordId());
        getTopPatientInfoDTO.setPatientAge(DateUtil.yearCompare(getTopPatientInfoDTO.getBirthday(), DateUtil.now()));
        return getTopPatientInfoDTO;
    }

    /**
     * 根据ids获取病人信息
     *
     * @param ids 病人ids
     * @return 病人信息
     */
    public Map<Long, PatientInfo> patientInfoMapByIds(List<Long> ids) {
        Map<Long, PatientInfo> patientInfoMap = this.listByIds(ids).stream()
                .filter(i -> i.getIsDeleted().equals("N")).collect(Collectors.toMap(PatientInfo::getId, i -> i));
        return patientInfoMap;
    }

    /**
     * 患者登录
     *
     * @param signInVO
     * @return
     */
    public List<SignInDTO> signIn(SignInVO signInVO) {
        String time = DateUtil.formatDateTime(new Date());
        List<SignInDTO> list = Lists.newArrayList();
        list.add(new SignInDTO("A001", "浙江大学医学院附属邵逸夫医院",
                "D01", "全科", "YS001", "付医生",
                "1600", "王明明", time, "4468352", "15"));
        list.add(new SignInDTO("A001", "浙江大学医学院附属邵逸夫医院",
                "D02", "儿科", "YS001", "付医生",
                "1600", "王明明", time, "4468352", "18"));
        if (signInVO.getPatientInfoType().equals("101")) {
            if (signInVO.getPatientInfo().equals("10123439991230120X")) {
                list.remove(1);
            } else if (signInVO.getPatientInfo().equals("666666180002301234")) {
                list.clear();
            } else if (signInVO.getPatientInfo().equals("100000201901010000")) {
                return null;
            }
        }

        return list;
    }


}
