package com.diagbot.client;

import com.diagbot.client.bean.ResponseData;
import com.diagbot.client.bean.SearchData;
import com.diagbot.client.hystrix.AiptServiceHystrix;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptRetrievalDTO;
import com.diagbot.dto.PartSymptomDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.VersionWrapperDTO;
import com.diagbot.vo.ConceptUsualVO;
import com.diagbot.vo.PartSymptomVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.VersionVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description: 调用知识库服务
 * @author: gaodm
 * @time: 2019/2/1 17:00
 */
@FeignClient(value = "aipt-service", fallback = AiptServiceHystrix.class)
public interface AiptServiceClient {

    @PostMapping(value = "/part/getSymptomByPartName")
    RespDTO<List<PartSymptomDTO>> getSymptomByPartName(@RequestBody PartSymptomVO partSymptomVO);

    /**
     * 获取常用标签
     *
     * @param conceptUsualVO
     * @return
     */
    @PostMapping(value = "/concept/getConceptUsual")
    RespDTO<List<ConceptBaseDTO>> getConceptUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO);

    /**
     * 标签检索
     *
     * @param retrievalVO
     * @return
     */
    @PostMapping("/concept/retrivelConceptInfo")
    RespDTO<List<ConceptRetrievalDTO>> retrivelConceptInfo(@RequestBody @Valid RetrievalVO retrievalVO);

    @PostMapping(value = "/clinicaldata/processData")
    RespDTO<ResponseData> aiptData(@RequestBody SearchData searchData);

    /**
     * 获取版本信息
     */
    @PostMapping("/versionInfo/getVersionInfoAlls")
    RespDTO<VersionWrapperDTO> getVersionInfoAlls(@RequestBody VersionVO versionVO);

}