package com.diagbot.facade;

import com.diagbot.client.AiptServiceClient;
import com.diagbot.client.TranServiceClient;
import com.diagbot.client.bean.FeatureRate;
import com.diagbot.client.bean.ResponseData;
import com.diagbot.client.bean.SearchData;
import com.diagbot.dto.AIDTO;
import com.diagbot.dto.HospitalDeptDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.FastJsonUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.AIVO;
import com.diagbot.vo.HospitalDeptVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 调用AI服务业务实现
 * @author: gaodm
 * @time: 2018/10/25 10:09
 */
@Component
public class AIFacade {
    @Autowired
    private AiptServiceClient aiptServiceClient;
    @Autowired
    private TranServiceClient tranServiceClient;

    /**
     * 调用ai接口业务逻辑
     *
     * @param aivo 搜索参数
     * @return AI接口返回结果
     */
    public AIDTO push(AIVO aivo, String type) {
        SearchData searchData = new SearchData();
        searchData.setAge(aivo.getAge());
        searchData.setSymptom(aivo.getSymptom());
        switch (aivo.getSex()) {
            case 1:
                searchData.setSex("M");
                break;
            case 2:
                searchData.setSex("F");
                break;
            default:
                throw new CommonException(CommonErrorCode.PARAM_ERROR, "不能输入男女以外的性别");
        }
        searchData.setFeatureType(type);
        searchData.setSysCode("1");
        switch (type) {
            case "1":
                searchData.setLength(10);
                break;
            case "7":
                searchData.setLength(10);
                break;
        }
        RespDTO<ResponseData> res = aiptServiceClient.aiptData(searchData);
        RespDTOUtil.respNGDealCover(res, "中间层没有结果返回");
        AIDTO aidto = new AIDTO();
        switch (type) {
            case "1":
                aidto.setItems(res.data.getSymptom());
                break;
            case "7":
                List<FeatureRate> items = new ArrayList<>();
                if (ListUtil.isNotEmpty(res.data.getDis())) {
                    Map<String, List<FeatureRate>> disFeatureMap = new LinkedHashMap<>();
                    //诊断分类
                    for (FeatureRate featureRate : res.data.getDis()) {
                        if (StringUtil.isBlank(featureRate.getDesc())) {
                            featureRate.setDesc("{\"可能诊断\":\"\"}");
                        }
                        Map<String, Object> descMap = FastJsonUtils.getJsonToMap(featureRate.getDesc());
                        for (String disClass : descMap.keySet()) {
                            List<FeatureRate> featureRateList = Lists.newLinkedList();
                            if (disFeatureMap.get(disClass) != null) {
                                featureRateList = disFeatureMap.get(disClass);
                            }
                            featureRateList.add(featureRate);
                            disFeatureMap.put(disClass, featureRateList);
                        }
                    }

                    FeatureRate item;
                    if (disFeatureMap.get("确诊") != null) {
                        items = getItems(disFeatureMap, "确诊", 1);
                    }
                    if (disFeatureMap.get("拟诊") != null && ListUtil.isEmpty(items)) {
                        items = getItems(disFeatureMap, "拟诊", 2);
                    }
                    if (disFeatureMap.get("可能诊断") != null && ListUtil.isEmpty(items)) {
                        Double rate;
                        Map<String, Double> map = new HashMap<>();
                        List<FeatureRate> featureRateList = disFeatureMap.get("可能诊断");
                        featureRateList = featureRateList
                                .stream()
                                .filter(i -> i.getExtraProperty() != null)
                                .filter(i -> i.getRate() != null)
                                .collect(Collectors.toList());
                        for (FeatureRate featureRate : featureRateList) {
                            if (featureRate.getRate() != null && Double.valueOf(featureRate.getRate()) > 0.05d) {
                                if (StringUtil.isNotBlank(featureRate.getExtraProperty())) {
                                    if (null != map.get(featureRate.getExtraProperty())) {
                                        rate = map.get(featureRate.getExtraProperty())
                                                + Double.valueOf(featureRate.getRate());
                                        map.replace(featureRate.getExtraProperty(), rate);
                                    } else {
                                        map.put(featureRate.getExtraProperty(), Double.valueOf(featureRate.getRate()));
                                    }
                                }
                            }
                        }
                        List<Map.Entry<String, Double>> list = new ArrayList<Map.Entry<String, Double>>(map.entrySet());
                        Collections.sort(list, new Comparator<Map.Entry<String, Double>>() {
                            //降序排序
                            public int compare(Map.Entry<String, Double> o1,
                                               Map.Entry<String, Double> o2) {
                                return o2.getValue().compareTo(o1.getValue());
                            }
                        });
                        for (Map.Entry<String, Double> entry : list) {
                            if (items.size() < 2) {
                                item = new FeatureRate();
                                item.setExtraProperty(entry.getKey());
                                items.add(item);
                            } else {
                                break;
                            }
                        }
                        if (ListUtil.isEmpty(items)) {
                            item = new FeatureRate();
                            item.setExtraProperty(res.data.getDis().get(0).getExtraProperty());
                            items.add(item);
                        }
                    }
                }
                if (StringUtil.isNotBlank(aivo.getHospitalCode())) {
                    //获取对接科室
                    if (ListUtil.isNotEmpty(items)) {
                        HospitalDeptVO hospitalDeptVO = new HospitalDeptVO();
                        hospitalDeptVO.setHospitalCode(aivo.getHospitalCode());
                        hospitalDeptVO.setConceptNames(items
                                .stream()
                                .map(i -> i.getExtraProperty())
                                .collect(Collectors.toList()));
                        RespDTO<Map<String, List<HospitalDeptDTO>>> retDeptMap
                                = tranServiceClient.hospitalDeptByConceptNames(hospitalDeptVO);
                        if (RespDTOUtil.respIsOK(retDeptMap)) {
                            Map<String, List<HospitalDeptDTO>> deptMap = retDeptMap.data;
                            for (FeatureRate item : items) {
                                item.setHospitalDepts(deptMap.get(item.getExtraProperty()));
                            }
                        }
                    }
                }
                aidto.setItems(items);
                break;
        }
        return aidto;
    }

    private List<FeatureRate> getItems(Map<String, List<FeatureRate>> disFeatureMap,
                                       String type, int size) {
        List<FeatureRate> items = new ArrayList<>();

        List<FeatureRate> featureRateList = disFeatureMap.get(type);
        featureRateList = featureRateList
                .stream()
                .filter(i -> i.getExtraProperty() != null)
                .collect(Collectors.toList());
        if (ListUtil.isNotEmpty(featureRateList)) {
            Map<String, List<FeatureRate>> disMap
                    = EntityUtil.makeEntityListMap(featureRateList, "extraProperty");
            List<Map.Entry<String, List<FeatureRate>>> list = new ArrayList<Map.Entry<String, List<FeatureRate>>>(disMap.entrySet());
            Collections.sort(list, new Comparator<Map.Entry<String, List<FeatureRate>>>() {
                //降序排序
                public int compare(Map.Entry<String, List<FeatureRate>> o1,
                                   Map.Entry<String, List<FeatureRate>> o2) {
                    return Integer.compare(o2.getValue().size(), o1.getValue().size());
                }
            });
            FeatureRate item;
            for (Map.Entry<String, List<FeatureRate>> entry : list) {
                if (items.size() < size) {
                    item = new FeatureRate();
                    item.setExtraProperty(entry.getKey());
                    items.add(item);
                } else {
                    break;
                }
            }
        }

        return items;
    }
}